@@ -20,6 +20,8 @@
 #' Inference is conducted in a fully Bayesian setting using the integrated nested Laplace approximation (INLA; \insertCite{rue2009approximate;textual}{bigDM}) technique through the R-INLA package (\url{https://www.r-inla.org/}).
 #' For the scalable model proposals \insertCite{orozco2020}{bigDM}, approximate values of the Deviance Information Criterion (DIC) and Watanabe-Akaike Information Criterion (WAIC) can also be computed.
 #'
+#' The function allows also to use the new hybrid approximate method that combines the Laplace method with a low-rank Variational Bayes correction to the posterior mean \insertCite{vanNiekerk2023}{bigDM} by including the \code{inla.mode="compact"} argument.
+#'
 #' @details For a full model specification and further details see the vignettes accompanying this package.
 #'
 #' @references
@@ -39,6 +41,8 @@
 #'
 #' \insertRef{orozco2020}{bigDM}
 #'
+#' \insertRef{vanNiekerk2023}{bigDM}
+#'
 #' @param carto object of class \code{SpatialPolygonsDataFrame} or \code{sf}.
 #' This object must contain at least the target variables of interest specified in the arguments \code{ID.area}, \code{O} and \code{E}.
 #' @param ID.area character; name of the variable that contains the IDs of spatial areal units.
@@ -73,6 +77,8 @@
 #' @param plan one of either \code{"sequential"} or \code{"cluster"}, which specifies the computation strategy used for model fitting using the 'future' package.
 #' If \code{plan="sequential"} (default) the models are fitted sequentially and in the current R session (local machine). If \code{plan="cluster"} the models are fitted in parallel on external R sessions (local machine) or distributed in remote computing nodes.
 #' @param workers character or vector (default \code{NULL}) containing the identifications of the local or remote workers where the models are going to be processed. Only required if \code{plan="cluster"}.
+#' @param inla.mode one of either \code{"classic"} (default) or \code{"compact"}, which specifies the approximation method used by INLA. See \code{help(inla)} for further details.
+#' @param num.threads maximum number of threads the inla-program will use. See \code{help(inla)} for further details.
 #'
 #' @return This function returns an object of class \code{inla}. See the \code{\link{mergeINLA}} function for details.
 #'
@@ -84,7 +90,7 @@
 #'
 #' @examples
 #' \dontrun{
-#' 
+#'
 #' if(require("INLA", quietly=TRUE)){
 #'
 #'   ## Load the Spain colorectal cancer mortality data ##
@@ -123,10 +129,14 @@
 CAR_INLA <- function(carto=NULL, ID.area=NULL, ID.group=NULL, O=NULL, E=NULL, X=NULL, confounding=NULL,
                      W=NULL, prior="Leroux", model="partition", k=0, strategy="simplified.laplace",
                      PCpriors=FALSE, seed=NULL, n.sample=1000, compute.intercept=FALSE, compute.DIC=TRUE,
-                     save.models=FALSE, plan="sequential", workers=NULL, merge.strategy="original"){
+                     save.models=FALSE, plan="sequential", workers=NULL, merge.strategy="original",
+                     inla.mode="classic", num.threads=NULL){
 
   if(suppressPackageStartupMessages(requireNamespace("INLA", quietly=TRUE))){
 
+        ## Set the 'num.threads' argument ##
+        if(is.null(num.threads)) num.threads <- INLA::inla.getOption("num.threads")
+
         ## Check for errors ##
         if(is.null(carto))
                 stop("the 'carto' argument is missing")
@@ -241,7 +251,7 @@ CAR_INLA <- function(carto=NULL, ID.area=NULL, ID.group=NULL, O=NULL, E=NULL, X=
         formula <- stats::as.formula(form)
 
         ## Auxiliary functions ##
-        FitModels <- function(Rs, Rs.Leroux, data.INLA, d, D){
+        FitModels <- function(Rs, Rs.Leroux, data.INLA, d, D, ...){
 
                 cat(sprintf("+ Model %d of %d",d,D),"\n")
 
@@ -278,7 +288,7 @@ CAR_INLA <- function(carto=NULL, ID.area=NULL, ID.group=NULL, O=NULL, E=NULL, X=
                 models <- inla(formula, family="poisson", data=data.INLA, E=E,
                                control.predictor=list(compute=TRUE, link=1, cdf=c(log(1))),
                                control.compute=list(dic=TRUE, cpo=TRUE, waic=TRUE, config=TRUE, return.marginals.predictor=TRUE),
-                               control.inla=list(strategy=strategy))
+                               control.inla=list(strategy=strategy), ...)
                 return(models)
         }
 
@@ -288,8 +298,10 @@ CAR_INLA <- function(carto=NULL, ID.area=NULL, ID.group=NULL, O=NULL, E=NULL, X=
 
                 W <- aux$W
                 n <- nrow(W)
-                Rs <- inla.as.sparse(Diagonal(n,colSums(W))-W)
-                Rs.Leroux <- inla.as.sparse(Diagonal(n)-Rs)
+                Rs <- as(Diagonal(n,colSums(W))-W,"TsparseMatrix")
+                Rs.Leroux <- as(Diagonal(n)-Rs,"TsparseMatrix")
+                # Rs <- inla.as.sparse(Diagonal(n,colSums(W))-W)
+                # Rs.Leroux <- inla.as.sparse(Diagonal(n)-Rs)
 
                 data.INLA <- data.frame(O=data[,O], E=data[,E], Area=data[,ID.area], ID.area=seq(1,n), data[,X])
                 names(data.INLA)[grep("^data...",names(data.INLA))] <- X
@@ -297,7 +309,8 @@ CAR_INLA <- function(carto=NULL, ID.area=NULL, ID.group=NULL, O=NULL, E=NULL, X=
                 Model <- inla(formula, family="poisson", data=data.INLA, E=E,
                               control.predictor=list(compute=TRUE, link=1, cdf=c(log(1))),
                               control.compute=list(dic=TRUE, cpo=TRUE, waic=TRUE, config=TRUE, return.marginals.predictor=TRUE),
-                              control.inla=list(strategy=strategy))
+                              control.inla=list(strategy=strategy),
+                              inla.mode=inla.mode, num.threads=num.threads)
 
                 ## Alleviate spatial confounding ##
                 if(!is.null(confounding)){
@@ -328,7 +341,8 @@ CAR_INLA <- function(carto=NULL, ID.area=NULL, ID.group=NULL, O=NULL, E=NULL, X=
                                               control.predictor=list(compute=TRUE, link=1, cdf=c(log(1))),
                                               control.compute=list(dic=TRUE, cpo=TRUE, waic=TRUE, config=TRUE, return.marginals.predictor=TRUE),
                                               lincomb=beta.lc,
-                                              control.inla=list(strategy=strategy))
+                                              control.inla=list(strategy=strategy),
+                                              inla.mode=inla.mode, num.threads=num.threads)
                                 cat("Done \n")
 
                                 names <- rownames(Model$summary.fixed)
@@ -355,7 +369,8 @@ CAR_INLA <- function(carto=NULL, ID.area=NULL, ID.group=NULL, O=NULL, E=NULL, X=
                                 Model <- inla(formula, family="poisson", data=data.INLA, E=E,
                                               control.predictor=list(compute=TRUE, link=1, cdf=c(log(1))),
                                               control.compute=list(dic=TRUE, cpo=TRUE, waic=TRUE, config=TRUE, return.marginals.predictor=TRUE),
-                                              control.inla=list(strategy=strategy))
+                                              control.inla=list(strategy=strategy),
+                                              inla.mode=inla.mode, num.threads=num.threads)
 
                                 t.eigen <- 0
                         }
@@ -402,16 +417,17 @@ CAR_INLA <- function(carto=NULL, ID.area=NULL, ID.group=NULL, O=NULL, E=NULL, X=
                 D <- length(data.INLA)
 
                 if(plan=="sequential"){
-                        inla.models <- mapply(FitModels, Rs=Rs, Rs.Leroux=Rs.Leroux, data.INLA=data.INLA, d=seq(1,D), D=D, SIMPLIFY=FALSE)
+                        inla.models <- mapply(FitModels, Rs=Rs, Rs.Leroux=Rs.Leroux, data.INLA=data.INLA, d=seq(1,D), D=D, num.threads=num.threads, inla.mode=inla.mode, SIMPLIFY=FALSE)
                 }
 
                 if(plan=="cluster"){
                         cl <- future::makeClusterPSOCK(workers, revtunnel=TRUE, outfile="")
-                        oplan <- future::plan(list(future::tweak(cluster, workers=workers), multisession))
+                        oplan <- future::plan(cluster, workers=cl)
+                        # oplan <- future::plan(list(future::tweak(cluster, workers=workers), multisession))
                         on.exit(future::plan(oplan))
 
                         cpu.time <- system.time({
-                                inla.models <- future.apply::future_mapply(FitModels, Rs=Rs, Rs.Leroux=Rs.Leroux, data.INLA=data.INLA, d=seq(1,D), D=D, SIMPLIFY=FALSE, future.seed=TRUE)
+                                inla.models <- future.apply::future_mapply(FitModels, Rs=Rs, Rs.Leroux=Rs.Leroux, data.INLA=data.INLA, d=seq(1,D), D=D, num.threads=num.threads, inla.mode=inla.mode, future.seed=TRUE, SIMPLIFY=FALSE)
                         })
 
                         stopCluster(cl)
@@ -439,8 +455,9 @@ CAR_INLA <- function(carto=NULL, ID.area=NULL, ID.group=NULL, O=NULL, E=NULL, X=
         return(Model)
 
   }else{
-        stop("\nINLA library is not installed!\nPlease use following command to install the stable version of the R-INLA package:\n\ninstall.packages('INLA', repos=c(getOption('repos'), INLA='https://inla.r-inla-download.org/R/stable'), dep=TRUE)")
+        stop("\nINLA library is not installed!\nPlease use the following command to install the stable version of the R-INLA package:\n\ninstall.packages('INLA', repos=c(getOption('repos'), INLA='https://inla.r-inla-download.org/R/stable'), dep=TRUE)")
   }
 }
 
 utils::globalVariables(c("inla.as.sparse"))
+utils::globalVariables(c("inla.setOption"))
