@@ -16,6 +16,8 @@
 #' Inference is conducted in a fully Bayesian setting using the integrated nested Laplace approximation (INLA; \insertCite{rue2009approximate;textual}{bigDM}) technique through the R-INLA package (\url{https://www.r-inla.org/}).
 #' For the scalable model proposals \insertCite{orozco2022}{bigDM}, approximate values of the Deviance Information Criterion (DIC) and Watanabe-Akaike Information Criterion (WAIC) can also be computed.
 #'
+#' The function allows also to use the new hybrid approximate method that combines the Laplace method with a low-rank Variational Bayes correction to the posterior mean \insertCite{vanNiekerk2023}{bigDM} by including the \code{inla.mode="compact"} argument.
+#'
 #' @details For a full model specification and further details see the vignettes accompanying this package.
 #'
 #' @references
@@ -27,6 +29,8 @@
 #'
 #' \insertRef{orozco2022}{bigDM}
 #'
+#' \insertRef{vanNiekerk2023}{bigDM}
+#'
 #' @param carto object of class \code{SpatialPolygonsDataFrame} or \code{sf}. This object must contain at least the variable with the identifiers of the spatial areal units specified in the argument \code{ID.area}.
 #' @param data object of class \code{data.frame} that must contain the target variables of interest specified in the arguments \code{ID.area}, \code{ID.year}, \code{O} and \code{E}.
 #' @param ID.area character; name of the variable that contains the IDs of spatial areal units. The values of this variable must match those given in the \code{carto} and \code{data} variable.
@@ -41,6 +45,8 @@
 #' @param model one of either \code{"global"} or \code{"partition"} (default), which specifies the \emph{Global model} or one of the scalable model proposal's (\emph{Disjoint model} and \emph{k-order neighbourhood model}, respectively).
 #' @param k numeric value with the neighbourhood order used for the partition model. Usually k=2 or 3 is enough to get good results. If k=0 (default) the \emph{Disjoint model} is considered. Only required if \code{model="partition"}.
 #' @param strategy one of either \code{"gaussian"}, \code{"simplified.laplace"} (default), \code{"laplace"} or \code{"adaptive"}, which specifies the approximation strategy considered in the \code{inla} function.
+#' @param scale.model logical value (default \code{NULL}); if \code{TRUE} the structure matrices of the model are scaled so their generalized variances are equal to 1.
+#' It only works if arguments \code{spatial="intrinsic"} or \code{spatial="BYM2"} are specified.
 #' @param PCpriors logical value (default \code{FALSE}); if \code{TRUE} then penalised complexity (PC) priors are used for the precision parameter of the spatial random effect.
 #' It only works if arguments \code{spatial="intrinsic"} or \code{spatial="BYM2"} are specified.
 #' @param seed numeric (default \code{NULL}); control the RNG of the \code{inla.qsample} function. See \code{help(inla.qsample)} for further information.
@@ -53,6 +59,8 @@
 #' @param plan one of either \code{"sequential"} or \code{"cluster"}, which specifies the computation strategy used for model fitting using the 'future' package.
 #' If \code{plan="sequential"} (default) the models are fitted sequentially and in the current R session (local machine). If \code{plan="cluster"} the models are fitted in parallel on external R sessions (local machine) or distributed in remote computing nodes.
 #' @param workers character or vector (default \code{NULL}) containing the identifications of the local or remote workers where the models are going to be processed. Only required if \code{plan="cluster"}.
+#' @param inla.mode one of either \code{"classic"} (default) or \code{"compact"}, which specifies the approximation method used by INLA. See \code{help(inla)} for further details.
+#' @param num.threads maximum number of threads the inla-program will use. See \code{help(inla)} for further details.
 #'
 #' @return This function returns an object of class \code{inla}. See the \code{\link{mergeINLA}} function for details.
 #'
@@ -103,11 +111,15 @@
 STCAR_INLA <- function(carto=NULL, data=NULL, ID.area=NULL, ID.year=NULL, ID.group=NULL, O=NULL, E=NULL,
                        W=NULL, spatial="Leroux", temporal="rw1", interaction="TypeIV",
                        model="partition", k=0, strategy="simplified.laplace",
-                       PCpriors=FALSE, seed=NULL, n.sample=1000, compute.intercept=FALSE, compute.DIC=TRUE,
-                       save.models=FALSE, plan="sequential", workers=NULL, merge.strategy="original"){
+                       scale.model=NULL, PCpriors=FALSE, seed=NULL, n.sample=1000, compute.intercept=FALSE, compute.DIC=TRUE,
+                       save.models=FALSE, plan="sequential", workers=NULL, merge.strategy="original",
+                       inla.mode="classic", num.threads=NULL){
 
   if(suppressPackageStartupMessages(requireNamespace("INLA", quietly=TRUE))){
 
+        ## Set the 'num.threads' argument ##
+        if(is.null(num.threads)) num.threads <- INLA::inla.getOption("num.threads")
+
         ## Check for errors ##
         if(is.null(carto))
                 stop("the 'carto' argument is missing")
@@ -209,12 +221,22 @@ STCAR_INLA <- function(carto=NULL, data=NULL, ID.area=NULL, ID.year=NULL, ID.gro
                         if(temporal=="rw2") r.def <- 2*S
                         A.constr <- kronecker(matrix(1,1,T),Diagonal(S))
                         A.constr <- as(A.constr[-1,],"matrix")
+
+                        if(PCpriors | !is.null(scale.model)){
+                                sigma.ref <- exp(mean(log(diag(MASS::ginv(as.matrix(Rt))))))
+                                R <- R*sigma.ref
+                        }
                 }
                 if(interaction=="TypeIII"){
                         R <- kronecker(Diagonal(T),Rs)
                         r.def <- T
                         A.constr <- kronecker(Diagonal(T),matrix(1,1,S))
                         A.constr <- as(A.constr[-1,],"matrix")
+
+                        if(PCpriors | !is.null(scale.model)){
+                                sigma.ref <- exp(mean(log(diag(MASS::ginv(as.matrix(Rs))))))
+                                R <- R*sigma.ref
+                        }
                 }
                 if(interaction=="TypeIV"){
                         R <- kronecker(Rt,Rs)
@@ -223,12 +245,11 @@ STCAR_INLA <- function(carto=NULL, data=NULL, ID.area=NULL, ID.year=NULL, ID.gro
                         A1 <- kronecker(matrix(1,1,T),Diagonal(S))
                         A2 <- kronecker(Diagonal(T),matrix(1,1,S))
                         A.constr <- as(rbind(A1[-1,],A2[-1,]),"matrix")
-                }
-
-                if(interaction!="TypeI" & PCpriors){
-                        sigma.ref1 <- exp(mean(log(diag(MASS::ginv(as.matrix(Rs))))))
-                        sigma.ref2 <- exp(mean(log(diag(MASS::ginv(as.matrix(Rt))))))
-                        R <- R*sigma.ref1*sigma.ref2
+                        if(PCpriors | !is.null(scale.model)){
+                                sigma.ref1 <- exp(mean(log(diag(MASS::ginv(as.matrix(Rs))))))
+                                sigma.ref2 <- exp(mean(log(diag(MASS::ginv(as.matrix(Rt))))))
+                                R <- R*sigma.ref1*sigma.ref2
+                        }
                 }
 
                 return(list(R=R,r.def=r.def,A.constr=A.constr))
@@ -241,7 +262,7 @@ STCAR_INLA <- function(carto=NULL, data=NULL, ID.area=NULL, ID.year=NULL, ID.gro
                 form <- paste(form,"f(ID.area, model='generic1', Cmatrix=Rs.Leroux, constr=TRUE, hyper=list(prec=list(prior=sdunif),beta=list(prior=lunif, initial=0)))", sep="")
         }
         if(spatial=="intrinsic" & !PCpriors) {
-                form <- paste(form,"f(ID.area, model='besag', graph=Rs, constr=TRUE, hyper=list(prec=list(prior=sdunif)))", sep="")
+                form <- paste(form,"f(ID.area, model='besag', graph=Rs, constr=TRUE, scale.model=scale.model, hyper=list(prec=list(prior=sdunif)))", sep="")
         }
         if(spatial=="intrinsic" & PCpriors) {
                 form <- paste(form,"f(ID.area, model='besag', graph=Rs, constr=TRUE, scale.model=TRUE, hyper=list(prec=list(prior='pc.prec', param=c(1,0.01))))", sep="")
@@ -250,26 +271,26 @@ STCAR_INLA <- function(carto=NULL, data=NULL, ID.area=NULL, ID.year=NULL, ID.gro
                 form <- paste(form,"f(ID.area, model='bym', graph=Rs, constr=TRUE, hyper=list(theta1=list(prior=sdunif), theta2=list(prior=sdunif)))", sep="")
         }
         if(spatial=="BYM2" & !PCpriors) {
-                form <- paste(form,"f(ID.area, model='bym2', graph=Rs, constr=TRUE, hyper=list(prec=list(prior=sdunif),phi=list(prior=lunif, initial=0)))", sep="")
+                form <- paste(form,"f(ID.area, model='bym2', graph=Rs, constr=TRUE, scale.model=scale.model, hyper=list(prec=list(prior=sdunif),phi=list(prior=lunif, initial=0)))", sep="")
         }
         if(spatial=="BYM2" & PCpriors) {
                 form <- paste(form,"f(ID.area, model='bym2', graph=Rs, constr=TRUE, scale.model=TRUE, hyper=list(prec=list(prior='pc.prec', param=c(1,0.01)),phi=list(prior='pc', param=c(0.5,0.5))))", sep="")
         }
 
         if(temporal=="rw1" & !PCpriors) {
-                form <- paste(form,"+ f(ID.year, model='rw1', constr=TRUE, hyper=list(prec=list(prior=sdunif)))", sep="")
+                form <- paste(form,"+ f(ID.year, model='rw1', constr=TRUE, scale.model=scale.model, hyper=list(prec=list(prior=sdunif)))", sep="")
         }
         if(temporal=="rw1" & PCpriors) {
                 form <- paste(form,"+ f(ID.year, model='rw1', constr=TRUE, scale.model=TRUE, hyper=list(prec=list(prior='pc.prec', param=c(1,0.01))))", sep="")
         }
         if(temporal=="rw2" & interaction!="TypeII" & !PCpriors) {
-                form <- paste(form,"+ f(ID.year, model='rw2', constr=TRUE, hyper=list(prec=list(prior=sdunif)))", sep="")
+                form <- paste(form,"+ f(ID.year, model='rw2', constr=TRUE, scale.model=scale.model, hyper=list(prec=list(prior=sdunif)))", sep="")
         }
         if(temporal=="rw2" & interaction!="TypeII" & PCpriors) {
                 form <- paste(form,"+ f(ID.year, model='rw2', constr=TRUE, scale.model=TRUE, hyper=list(prec=list(prior='pc.prec', param=c(1,0.01))))", sep="")
         }
         if(temporal=="rw2" & interaction=="TypeII" & !PCpriors) {
-                form <- paste(form,"+ f(ID.year, model='rw2', constr=TRUE, hyper=list(prec=list(prior=sdunif)), extraconstr=list(A=matrix(1:T,1,T),e=0))", sep="")
+                form <- paste(form,"+ f(ID.year, model='rw2', constr=TRUE, scale.model=scale.model, hyper=list(prec=list(prior=sdunif)), extraconstr=list(A=matrix(1:T,1,T),e=0))", sep="")
         }
         if(temporal=="rw2" & interaction=="TypeII" & PCpriors) {
                 form <- paste(form,"+ f(ID.year, model='rw2', constr=TRUE, scale.model=TRUE, hyper=list(prec=list(prior='pc.prec', param=c(1,0.01))), extraconstr=list(A=matrix(1:T,1,T),e=0))", sep="")
@@ -297,10 +318,10 @@ STCAR_INLA <- function(carto=NULL, data=NULL, ID.area=NULL, ID.year=NULL, ID.gro
         formula <- stats::as.formula(form)
 
         ## Auxiliary functions ##
-        FitModels <- function(Rs, Rs.Leroux, R, r.def, A.constr, data.INLA, d, D){
+        FitModels <- function(Rs, Rs.Leroux, R, r.def, A.constr, data.INLA, d, D, ...){
 
                 cat(sprintf("+ Model %d of %d",d,D),"\n")
-          
+
                 Rs <- as(Rs,"TsparseMatrix")
                 Rs.Leroux <- as(Rs.Leroux,"TsparseMatrix")
                 # Rs <- inla.as.sparse(Rs)
@@ -313,7 +334,7 @@ STCAR_INLA <- function(carto=NULL, data=NULL, ID.area=NULL, ID.year=NULL, ID.gro
                         form <- paste(form,"f(ID.area, model='generic1', Cmatrix=Rs.Leroux, constr=TRUE, hyper=list(prec=list(prior=sdunif),beta=list(prior=lunif, initial=0)))", sep="")
                 }
                 if(spatial=="intrinsic" & !PCpriors) {
-                        form <- paste(form,"f(ID.area, model='besag', graph=Rs, constr=TRUE, hyper=list(prec=list(prior=sdunif)))", sep="")
+                        form <- paste(form,"f(ID.area, model='besag', graph=Rs, constr=TRUE, scale.model=scale.model, hyper=list(prec=list(prior=sdunif)))", sep="")
                 }
                 if(spatial=="intrinsic" & PCpriors) {
                         form <- paste(form,"f(ID.area, model='besag', graph=Rs, constr=TRUE, scale.model=TRUE, hyper=list(prec=list(prior='pc.prec', param=c(1,0.01))))", sep="")
@@ -322,26 +343,26 @@ STCAR_INLA <- function(carto=NULL, data=NULL, ID.area=NULL, ID.year=NULL, ID.gro
                         form <- paste(form,"f(ID.area, model='bym', graph=Rs, constr=TRUE, hyper=list(theta1=list(prior=sdunif), theta2=list(prior=sdunif)))", sep="")
                 }
                 if(spatial=="BYM2" & !PCpriors) {
-                        form <- paste(form,"f(ID.area, model='bym2', graph=Rs, constr=TRUE, hyper=list(prec=list(prior=sdunif),phi=list(prior=lunif, initial=0)))", sep="")
+                        form <- paste(form,"f(ID.area, model='bym2', graph=Rs, constr=TRUE, scale.model=scale.model, hyper=list(prec=list(prior=sdunif),phi=list(prior=lunif, initial=0)))", sep="")
                 }
                 if(spatial=="BYM2" & PCpriors) {
                         form <- paste(form,"f(ID.area, model='bym2', graph=Rs, constr=TRUE, scale.model=TRUE, hyper=list(prec=list(prior='pc.prec', param=c(1,0.01)),phi=list(prior='pc', param=c(0.5,0.5))))", sep="")
                 }
 
                 if(temporal=="rw1" & !PCpriors) {
-                        form <- paste(form,"+ f(ID.year, model='rw1', constr=TRUE, hyper=list(prec=list(prior=sdunif)))", sep="")
+                        form <- paste(form,"+ f(ID.year, model='rw1', constr=TRUE, scale.model=scale.model, hyper=list(prec=list(prior=sdunif)))", sep="")
                 }
                 if(temporal=="rw1" & PCpriors) {
                         form <- paste(form,"+ f(ID.year, model='rw1', constr=TRUE, scale.model=TRUE, hyper=list(prec=list(prior='pc.prec', param=c(1,0.01))))", sep="")
                 }
                 if(temporal=="rw2" & interaction!="TypeII" & !PCpriors) {
-                        form <- paste(form,"+ f(ID.year, model='rw2', constr=TRUE, hyper=list(prec=list(prior=sdunif)))", sep="")
+                        form <- paste(form,"+ f(ID.year, model='rw2', constr=TRUE, scale.model=scale.model, hyper=list(prec=list(prior=sdunif)))", sep="")
                 }
                 if(temporal=="rw2" & interaction!="TypeII" & PCpriors) {
                         form <- paste(form,"+ f(ID.year, model='rw2', constr=TRUE, scale.model=TRUE, hyper=list(prec=list(prior='pc.prec', param=c(1,0.01))))", sep="")
                 }
                 if(temporal=="rw2" & interaction=="TypeII" & !PCpriors) {
-                        form <- paste(form,"+ f(ID.year, model='rw2', constr=TRUE, hyper=list(prec=list(prior=sdunif)), extraconstr=list(A=matrix(1:T,1,T),e=0))", sep="")
+                        form <- paste(form,"+ f(ID.year, model='rw2', constr=TRUE, scale.model=scale.model, hyper=list(prec=list(prior=sdunif)), extraconstr=list(A=matrix(1:T,1,T),e=0))", sep="")
                 }
                 if(temporal=="rw2" & interaction=="TypeII" & PCpriors) {
                         form <- paste(form,"+ f(ID.year, model='rw2', constr=TRUE, scale.model=TRUE, hyper=list(prec=list(prior='pc.prec', param=c(1,0.01))), extraconstr=list(A=matrix(1:T,1,T),e=0))", sep="")
@@ -371,7 +392,7 @@ STCAR_INLA <- function(carto=NULL, data=NULL, ID.area=NULL, ID.year=NULL, ID.gro
                 models <- inla(formula, family="poisson", data=data.INLA, E=E,
                                control.predictor=list(compute=TRUE, link=1, cdf=c(log(1))),
                                control.compute=list(dic=TRUE, cpo=TRUE, waic=TRUE, config=TRUE, return.marginals.predictor=TRUE),
-                               control.inla=list(strategy=strategy))
+                               control.inla=list(strategy=strategy), ...)
                 return(models)
         }
 
@@ -396,7 +417,8 @@ STCAR_INLA <- function(carto=NULL, data=NULL, ID.area=NULL, ID.year=NULL, ID.gro
                 Model <- inla(formula, family="poisson", data=data.INLA, E=E,
                               control.predictor=list(compute=TRUE, link=1, cdf=c(log(1))),
                               control.compute=list(dic=TRUE, cpo=TRUE, waic=TRUE, config=TRUE, return.marginals.predictor=TRUE),
-                              control.inla=list(strategy=strategy))
+                              control.inla=list(strategy=strategy),
+                              inla.mode=inla.mode, num.threads=num.threads)
         }
 
         ## Partition model ##
@@ -418,7 +440,7 @@ STCAR_INLA <- function(carto=NULL, data=NULL, ID.area=NULL, ID.year=NULL, ID.gro
                                 cat(red("Running...\n"))
                         }
                 }
-                aux <- lapply(data.d, function(xx) aggregate(xx[,O], by=list(xx[,ID.area]), sum)$x)
+                aux <- lapply(data.d, function(xx) aggregate(xx[,O], by=list(xx[,ID.area]), sum, na.rm=T)$x)
                 prop.zero <- unlist(lapply(aux, function(x) mean(x==0)))
                 n.zero <- sum(prop.zero>0.5)
                 if(n.zero>0) fun()
@@ -438,16 +460,17 @@ STCAR_INLA <- function(carto=NULL, data=NULL, ID.area=NULL, ID.year=NULL, ID.gro
                 D <- length(data.INLA)
 
                 if(plan=="sequential"){
-                        inla.models <- mapply(FitModels, Rs=Rs, Rs.Leroux=Rs.Leroux, R=R, r.def=r.def, A.constr=A.constr, data.INLA=data.INLA, d=seq(1,D), D=D, SIMPLIFY=FALSE)
+                        inla.models <- mapply(FitModels, Rs=Rs, Rs.Leroux=Rs.Leroux, R=R, r.def=r.def, A.constr=A.constr, data.INLA=data.INLA, d=seq(1,D), D=D, num.threads=num.threads, inla.mode=inla.mode, SIMPLIFY=FALSE)
                 }
 
                 if(plan=="cluster"){
                         cl <- future::makeClusterPSOCK(workers, revtunnel=TRUE, outfile="")
-                        oplan <- future::plan(list(future::tweak(cluster, workers=workers), multisession))
+                        oplan <- future::plan(cluster, workers=cl)
+                        # oplan <- future::plan(list(future::tweak(cluster, workers=workers), multisession))
                         on.exit(future::plan(oplan))
 
                         cpu.time <- system.time({
-                                inla.models <- future.apply::future_mapply(FitModels, Rs=Rs, Rs.Leroux=Rs.Leroux, R=R, r.def=r.def, A.constr=A.constr, data.INLA=data.INLA, d=seq(1,D), D=D, SIMPLIFY=FALSE, future.seed=TRUE)
+                                inla.models <- future.apply::future_mapply(FitModels, Rs=Rs, Rs.Leroux=Rs.Leroux, R=R, r.def=r.def, A.constr=A.constr, data.INLA=data.INLA, d=seq(1,D), D=D,  num.threads=num.threads, inla.mode=inla.mode, future.seed=TRUE, SIMPLIFY=FALSE)
                         })
 
                         stopCluster(cl)
@@ -480,3 +503,4 @@ STCAR_INLA <- function(carto=NULL, data=NULL, ID.area=NULL, ID.year=NULL, ID.gro
 }
 
 # utils::globalVariables(c("inla.as.sparse"))
+utils::globalVariables(c("inla.setOption"))
